@@ -36,7 +36,7 @@ Example:
 """
 # %run __init__.py
 # %load_ext lab_black
-# TODO: update displayfile_definitions based on the extra work done...
+
 # +
 import pathlib
 import functools
@@ -45,7 +45,7 @@ from IPython.display import (
     display,
     clear_output,
     Markdown,
-)  # , Image JSON, HTML, IFrame,
+)
 import time
 import typing as ty
 import ipywidgets as w
@@ -53,7 +53,7 @@ import traitlets as tr
 from pydantic import BaseModel, validator, HttpUrl
 
 #  local imports
-from ipyautoui.autodisplayfile_renderers import (
+from ipyautoui.autodisplay_renderers import (
     DEFAULT_FILE_RENDERERS,
     handle_compound_ext,
 )
@@ -75,6 +75,8 @@ from ipyautoui.constants import (
     KWARGS_HOME_DISPLAY_FILES,
 )
 
+import requests
+
 # from mf library
 # try:
 #     from xlsxtemplater import from_excel
@@ -99,9 +101,22 @@ class DisplayObjectActions(BaseModel):
     ext: str = None
     name: str = None
     check_exists: ty.Callable = None
-    renderer: ty.Callable = lambda: print("renderer")
+    renderer: ty.Callable = None
     check_date_modified: ty.Callable = None
 
+    @validator("renderer", always=True)
+    def _renderer(cls, v, values):
+        if v is None:
+            ext = values["ext"]
+            map_ = values["map_renderers"]
+            if ext in map_.keys():
+                fn = functools.partial(map_[ext], values["path"])
+            else:
+                fn = lambda: w.HTML("File renderer not found")
+            return fn
+        else:
+            return functools.partial(v, values["path"])
+
     class Config:
         arbitrary_types_allowed = True
 
@@ -133,16 +148,6 @@ class DisplayFromPath(DisplayObjectActions):
             v = values["path"].name
         return v
 
-    @validator("renderer", always=True)
-    def _renderer(cls, v, values):
-        ext = values["ext"]
-        map_ = values["map_renderers"]
-        if ext in map_.keys():
-            fn = functools.partial(map_[ext], values["path"])
-        else:
-            fn = lambda: w.HTML("File renderer not found")
-        return fn
-
     @validator("ext", always=True)
     def _ext(cls, v, values):
         if values["path"] is not None:
@@ -161,8 +166,9 @@ class DisplayFromPath(DisplayObjectActions):
     def _date_modified(cls, v, values):
         p = values["path"]
         if p is not None:
-            fn = functools.partial(st_mtime_string, p)
-        return fn
+            return functools.partial(st_mtime_string, p)
+        else:
+            return None
 
     @validator("open_file", always=True)
     def _open_file(cls, v, values):
@@ -184,16 +190,50 @@ class DisplayFromPath(DisplayObjectActions):
         arbitrary_types_allowed = True
 
 
+def url_ok(url):
+
+    # exception block
+    try:
+
+        # pass the url into
+        # request.head
+        # response = requests.head(url)
+        # ^ TODO : why doens't this work?
+
+        response = requests.get(url)
+
+        # check the status code
+        if response.status_code == 200:
+            return True
+        else:
+            return False
+    except requests.ConnectionError as e:
+        return e
+
+
 # TODO: create a DisplayFromRequest actions. for use with API queries...?
 class DisplayFromRequest(DisplayObjectActions):
-    pass
+    path: HttpUrl
+
+    @validator("check_exists", always=True)
+    def _check_exists(cls, v, values):
+        fn = functools.partial(url_ok, values["path"])
+        return fn
+
+    @validator("name", always=True)
+    def _name(cls, v, values):
+        return values["path"].path
 
 
-# -
-
+# +
 # TODO: separate out the bit that is display data and display from path...
 # TODO: probs useful to have a `value` trait (allowing the object to be updated instead of remade)
 #       this probably means having DisplayObject as a base class and extending it for display file...
+
+ORDER_DEFAULT = ("exists", "openpreview", "openfile", "openfolder", "name")
+ORDER_NOTPATH = ("exists", "openpreview", "name")
+
+
 class DisplayObject(w.VBox):
     """
     class for displaying file-like objects.
@@ -210,7 +250,7 @@ class DisplayObject(w.VBox):
     @tr.validate("order")
     def _validate_order(self, proposal):
         for l in proposal["value"]:
-            if l not in self.default_order:
+            if l not in ORDER_DEFAULT:
                 raise ValueError(
                     """
                     order must include the following: ("exists", "openpreview", "openfile", "openfolder", "name")
@@ -238,12 +278,13 @@ class DisplayObject(w.VBox):
                 default is: ("exists", "openpreview", "openfile", "openfolder", "name")
                 reduce tuple to hide components
         """
-        self.default_order = ("exists", "openpreview", "openfile", "openfolder", "name")
         self.display_actions = display_actions
         self._init()
         self.auto_open = auto_open
-        if order is None:
-            self.order = self.default_order
+        if order is None and isinstance(display_actions.path, pathlib.Path):
+            self.order = ORDER_DEFAULT
+        elif order is None and not isinstance(display_actions.path, pathlib.Path):
+            self.order = ORDER_NOTPATH
         else:
             self.order = order
 
@@ -257,23 +298,38 @@ class DisplayObject(w.VBox):
         cls,
         path,
         newroot=pathlib.PureWindowsPath("J:/"),
-        file_renderers=None,
+        renderers=None,
         auto_open=False,
         order=None,
     ):
-        if file_renderers is not None:
-            file_renderers = merge_file_renderers(file_renderers)
+        if renderers is not None:
+            renderers = merge_file_renderers(renderers)
         else:
-            file_renderers = DEFAULT_FILE_RENDERERS
+            renderers = DEFAULT_FILE_RENDERERS
         display_actions = DisplayFromPath(
-            path=path, newroot=newroot, map_renderers=file_renderers
+            path=path, newroot=newroot, map_renderers=renderers
         )
         return cls(display_actions, auto_open=auto_open, order=order)
 
     # TODO: create a from_request classmethod
     @classmethod
-    def from_request(cls, path):
-        pass
+    def from_request(
+        cls,
+        path,
+        ext,
+        renderers=None,
+        auto_open=False,
+        order=None,
+    ):
+        if renderers is not None:
+            renderers = merge_file_renderers(renderers)
+        else:
+            renderers = DEFAULT_FILE_RENDERERS
+        order = ORDER_NOTPATH
+        display_actions = DisplayFromRequest(
+            path=path, ext=ext, map_renderers=renderers
+        )
+        return cls(display_actions, auto_open=auto_open, order=order)
 
     def tooltip_openpath(self, path):
         return str(make_new_path(path, newroot=self.display_actions.newroot))
@@ -365,6 +421,41 @@ open folder:
         self.out_caller.layout.display = "none"
 
 
+# -
+
+if __name__ == "__main__":
+    path = "https://catfact.ninja/fact"
+    ext = ".json"
+    display(DisplayFromRequest(path=path, ext=ext).renderer())
+
+if __name__ == "__main__":
+
+    path = "https://catfact.ninja/fact"
+    ext = ".json"
+    display(DisplayObject.from_request(path=path, ext=ext))
+
+if __name__ == "__main__":
+    import json
+    from datetime import datetime
+
+    def display_catfact(path):
+        di = json.loads(requests.get(path).content)
+        s = f"""
+🐱🐈😹 **CAT FACT** 😾🙀😿
+
+{di['fact']}
+
+*{datetime.now().strftime("%Y-%m-%d, %H:%M:%S")} - https://cat-fact.herokuapp.com/#/*"""
+        return Markdown(s)
+
+    path = "https://catfact.ninja/fact"
+    ext = ".catfact"
+    display(
+        DisplayObject.from_request(
+            path=path, ext=ext, renderers={".catfact": display_catfact}
+        )
+    )
+
 if __name__ == "__main__":
     from ipyautoui.test_schema import TestAutoLogic
     from ipyautoui.autoui import AutoUi
@@ -379,9 +470,11 @@ if __name__ == "__main__":
 
 
 if __name__ == "__main__":
-    d.order = "name"
+    d.order = (
+        "openpreview",
+        "name",
+    )
     d.auto_open = True
-    # d.show_exists = False
 
 if __name__ == "__main__":
     from ipyautoui.test_schema import TestAutoLogic
@@ -391,7 +484,7 @@ if __name__ == "__main__":
     )
     path1 = tests_constants.PATH_TEST_AUI
 
-    d = DisplayObject.from_path(path1, file_renderers=user_file_renderers)
+    d = DisplayObject.from_path(path1, renderers=user_file_renderers)
     display(d)
 
 
@@ -434,20 +527,17 @@ class AutoDisplay(tr.HasTraits):
     def __init__(
         self,
         display_objects_actions: ty.List[DisplayObjectActions],
-        patterns: ty.Union[str, ty.List] = None,
-        title: str = None,
+        patterns: ty.Union[str, ty.List, None] = None,
+        title: ty.Union[str, None] = None,
         display_showhide: bool = True,
     ):
         """
-
-
         Args:
             paths (ty.List[pathlib.Path]): list of paths to display
             default_file_renderers: default renderers
-            user_file_renderers: default = {}, custom user-defined file renderers
-            newroot: passed to open_file
             patterns: (str or list), patterns to auto-open
-            title: (str), dfeault = None,
+            title: (str), default = None,
+            display_showhide: bool = True,
 
 
         """
@@ -464,23 +554,24 @@ class AutoDisplay(tr.HasTraits):
     def from_paths(
         cls,
         paths: ty.List[pathlib.Path],
-        newroot=pathlib.PureWindowsPath("J:/"),
-        file_renderers=None,
+        newroot=pathlib.PureWindowsPath("J:/"),  # TODO: maproots
+        renderers=None,
         patterns: ty.Union[str, ty.List] = None,
-        title: str = None,
+        title: ty.Union[str, None] = None,
         display_showhide: bool = True,
     ):
-        if file_renderers is not None:
-            file_renderers = merge_file_renderers(file_renderers)
+        if renderers is not None:
+            renderers = merge_file_renderers(renderers)
         else:
-            file_renderers = DEFAULT_FILE_RENDERERS
+            renderers = DEFAULT_FILE_RENDERERS
         if not isinstance(paths, list):
             paths = [pathlib.Path(paths)]
 
-        display_objects_actions = [
-            DisplayFromPath(path=path, newroot=newroot, map_renderers=file_renderers)
-            for path in paths
-        ]
+        display_objects_actions = cls.actions_from_paths(
+            paths=paths,
+            newroot=newroot,
+            renderers=renderers,
+        )
         return cls(
             display_objects_actions,
             patterns=patterns,
@@ -488,6 +579,34 @@ class AutoDisplay(tr.HasTraits):
             display_showhide=display_showhide,
         )
 
+    @staticmethod
+    def actions_from_paths(
+        paths: ty.List[pathlib.Path],
+        newroot=pathlib.PureWindowsPath("J:/"),
+        renderers=None,
+    ):
+        return [
+            DisplayFromPath(path=path, newroot=newroot, map_renderers=renderers)
+            for path in paths
+        ]
+
+    def add_from_paths(
+        self,
+        paths,
+        newroot=pathlib.PureWindowsPath("J:/"),
+        renderers=None,
+    ):
+        if renderers is not None:
+            renderers = merge_file_renderers(renderers)
+        else:
+            renderers = DEFAULT_FILE_RENDERERS
+        paths = [p for p in paths if p not in self.paths]
+        _new_actions = self.actions_from_paths(
+            paths=paths, newroot=newroot, renderers=renderers
+        )
+        actions = self.display_objects_actions + _new_actions
+        self.display_objects_actions = actions
+
     @property
     def title(self):
         return self._title
@@ -521,9 +640,7 @@ class AutoDisplay(tr.HasTraits):
 
     @property
     def paths(self):
-        return [
-            wcPath(d.path) for d in self.display_objects_actions
-        ]  # self._paths  # [d.path for d in self.display_objects]
+        return [wcPath(d.path) for d in self.display_objects_actions]
 
     @property
     def display_objects_actions(self):
@@ -607,10 +724,7 @@ class AutoDisplay(tr.HasTraits):
 
 
 # +
-# TODO: render markdown to html using pandoc and rebase relative paths - https://github.com/jgm/pandoc/issues/3752
-# TODO: render Vega updating the data path
 # TODO: render pdf update the relative path
-# TODO: figure out why the spacing between rows is weird
 
 if __name__ == "__main__":
     from ipyautoui.test_schema import TestAutoLogic
@@ -634,8 +748,34 @@ if __name__ == "__main__":
 
     test_ui = AutoDisplay.from_paths(
         paths=[tests_constants.PATH_TEST_AUI],
-        file_renderers=user_file_renderers,
+        renderers=user_file_renderers,
         display_showhide=False,
     )
 
     display(test_ui)
+
+if __name__ == "__main__":
+    import json
+    from datetime import datetime
+
+    def display_catfact(path):
+        di = json.loads(requests.get(path).content)
+        s = f"""
+🐱🐈😹 **CAT FACT** 😾🙀😿
+
+{di['fact']}
+
+*{datetime.now().strftime("%Y-%m-%d, %H:%M:%S")} - https://cat-fact.herokuapp.com/#/*"""
+        return Markdown(s)
+
+    path = "https://catfact.ninja/fact"
+    ext = ".catfact"
+    d1 = DisplayFromRequest(path=path, ext=ext, renderer=display_catfact)
+
+    path = "https://official-joke-api.appspot.com/random_joke"
+    ext = ".json"
+    d2 = DisplayFromRequest(path=path, ext=ext)
+
+    test_display = AutoDisplay([d1, d2])
+    display(Markdown("### From requests: "))
+    display(test_display)
