@@ -13,7 +13,6 @@
 #     name: python3
 # ---
 
-# +
 """autoui is used to automatically create ipywidget user input (UI) form from a pydantic schema.
 
 This module maps the pydantic fields to appropriate widgets based on type to display the data in the UI.
@@ -34,26 +33,18 @@ import functools
 import ipywidgets as w
 from IPython.display import display
 import traitlets as tr
-
-# TODO: Tasks pending completion -@jovyan at 7/18/2022, 2:07:55 PM
-# use traitlets_paths or not... pull request to main traitlets?
 import typing as ty
-from ipyautoui.constants import DOWNARROW_BUTTON_KWARGS
-from ipyautoui.custom.showhide import ShowHide
-import ipyautoui.automapschema as aumap
-import immutables
 import inspect
-from ipyautoui._utils import obj_from_importstr
 import json
+from IPython.display import display, clear_output
+import ipyautoui.automapschema as aumap
 from ipyautoui.constants import BUTTON_WIDTH_MIN
-from IPython.display import display, Markdown, clear_output, display_pretty
-from ipyautoui._utils import display_python_string
-from ipyautoui.custom.save_buttonbar import SaveButtonBar, SaveActions
-
-frozenmap = immutables.Map
+from ipyautoui._utils import display_python_string, obj_from_importstr
+from ipyautoui.custom.buttonbars import SaveButtonBar
+from ipyautoui.custom.showhide import ShowHide
+from ipyautoui.autowidgets import Nullable
 
 
-# +
 def _init_widgets_and_labels(
     pr: ty.Dict,
 ) -> tuple[ty.Dict[str, w.HBox], ty.Dict]:
@@ -79,27 +70,6 @@ def _init_widgets_and_labels(
     return di_labels, di_widgets
 
 
-def _init_model_schema(schema, by_alias=False):
-    if type(schema) == dict:
-        model = None  # jsonschema_to_pydantic(schema)
-        # IDEA: Possible implementations -@jovyan at 8/24/2022, 12:05:02 PM
-        # jsonschema_to_pydantic
-        # https://koxudaxi.github.io/datamodel-code-generator/using_as_module/
-    else:
-        model = schema  # the "model" passed is a pydantic model
-        schema = model.schema(by_alias=by_alias).copy()
-
-    if "definitions" in schema.keys():
-        li = [l for l in schema.keys() if l != "definitions"]
-        li = ["definitions"] + li
-        schema = {l: schema[l] for l in li}
-    # ^ put definitions at the top of the schema.
-    # otherwise definitions with $ref's in get copied in to the code
-
-    schema = aumap.attach_schema_refs(schema)
-    return model, schema
-
-
 # + tags=[]
 def _get_value_trait(obj_with_traits):
     """gets the trait type for a given object (looks for "_value" and
@@ -302,12 +272,16 @@ class AutoObjectFormLayout(AutoObjectShowRaw):
     def _observe_show_savebuttonbar(self, change):
         show_hide_widget(self.savebuttonbar, self.show_savebuttonbar)
 
-    def __init__(self):
+    def __init__(self, fns_onsave=None, fns_onrevert=None):
         super().__init__()
         self._init_form()
         self._init_bn_showraw_controls()
         self.fn_onshowraw = self.display_showraw
         self.fn_onhideraw = self.display_ui
+        if fns_onsave is not None:
+            self.fns_onsave = fns_onsave
+        if fns_onrevert is not None:
+            self.fns_onrevert = fns_onrevert
 
     def _init_form(self):
         self.autowidget = w.VBox()
@@ -333,10 +307,36 @@ class AutoObjectFormLayout(AutoObjectShowRaw):
         self.autowidget.layout.display = "None"
         return '{"test": "json"}'
 
+    @property
+    def fns_onsave(self):
+        return self.savebuttonbar.fns_onsave
+
+    @fns_onsave.setter
+    def fns_onsave(self, value):
+        if isinstance(value, list):
+            [self.savebuttonbar.fns_onsave_add_action(v) for v in value]
+        elif isinstance(value, ty.Callable):
+            self.savebuttonbar.fns_onrevert_add_action(value)
+        else:
+            raise ValueError("fns_onsave must be a callable or list of callables")
+
+    @property
+    def fns_onrevert(self):
+        return self.savebuttonbar.fns_onrevert
+
+    @fns_onrevert.setter
+    def fns_onrevert(self, value):
+        if isinstance(value, list):
+            [self.savebuttonbar.fns_onrevert_add_action(v) for v in value]
+        elif isinstance(value, ty.Callable):
+            self.savebuttonbar.fns_onrevert_add_action(value)
+        else:
+            raise ValueError("fns_onrevert must be a callable or list of callables")
+
 
 def demo_autoobject_form(title="test", description="a description of the title"):
     """for docs and testing only..."""
-    from ipyautoui.custom.save_buttonbar import SaveButtonBar
+    from ipyautoui.custom.buttonbars import SaveButtonBar
 
     form = AutoObjectFormLayout()
     form.title.value = make_bold(title)
@@ -425,7 +425,7 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
                         "if you want to use order to hide rows then set:"
                         "`order_hides_rows = True`"
                     )
-        # TODO: order not updating
+        # BUG: order not updating
         return v
 
     @tr.validate("order_can_hide_rows")
@@ -473,9 +473,12 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
     @value.setter
     def value(self, value):
         """this is for setting the value via the API"""
-        self._value = value
-        if hasattr(self, "di_widgets"):
-            self._update_widgets_from_value()
+        if value is None:
+            pass
+        else:
+            self._value = value
+            if hasattr(self, "di_widgets"):
+                self._update_widgets_from_value()
 
     def __init__(
         self,
@@ -486,9 +489,11 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
         fdir=None,
         order=None,
         insert_rows=None,
-        nested_widgets=None,
+        nested_widgets=None,  # TODO
         show_description: bool = True,
         show_title: bool = True,
+        fns_onsave=None,
+        fns_onrevert=None,
     ):
         """creates a widget input form from schema. datatype must be "object"
 
@@ -506,7 +511,7 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
         Returns:
             AutoObject(w.VBox)
         """
-        super().__init__()
+        super().__init__(fns_onsave=fns_onsave, fns_onrevert=fns_onrevert)
         self.insert_rows = insert_rows
         self.update_map_widgets = update_map_widgets
         self.fdir = fdir
@@ -546,7 +551,7 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
         self._init_controls()
 
     def _init_schema(self, schema, by_alias=False):
-        self.model, self.schema = _init_model_schema(schema, by_alias=by_alias)
+        self.model, self.schema = aumap._init_model_schema(schema, by_alias=by_alias)
         # if "type" not in self.schema.keys() and self.schema["type"] != "object":
         #     raise ValueError(
         #         '"type" must be in schema keys and "type" must == "object"'
@@ -565,6 +570,12 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
             for v in self.pr.values():
                 v = add_fdir_to_widgetcaller(v, self.fdir)
 
+    # def is_nullable(self, key):
+    #     if "nullable" in self.pr[key].keys() and self.pr[key]["nullable"]:
+    #         return True
+    #     else:
+    #         return False
+
     def _init_widgets(self):
         self.di_labels, self.di_widgets = _init_widgets_and_labels(self.pr)
         # self._value = self.di_widgets_value
@@ -630,12 +641,12 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
         self.savebuttonbar.unsaved_changes = True
         # NOTE: it is required to set the whole "_value" otherwise
         #       traitlets doesn't register the change.
-        #       -@jovyan at 7/18/2022, 12:45:48 PM
+        #       -@jovyan at 7/18/2022, 12:45:48 PMsavebuttonbar
 
     def _update_widgets_from_value(self):
         for k, v in self.value.items():
             if k in self.di_widgets.keys():
-                if v is None:
+                if v is None and not isinstance(self.di_widgets[k], Nullable):
                     v = _get_value_trait(self.di_widgets[k]).default()
                 self.di_widgets[k].value = v
             else:
@@ -665,15 +676,10 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
 
 
 if __name__ == "__main__":
-    from ipyautoui.constants import load_test_constants
-    from ipyautoui.test_schema import TestAutoLogicSimple
+    from ipyautoui.demo_schemas import CoreIpywidgets
 
-    test_constants = load_test_constants()
-    test = TestAutoLogicSimple()
-    schema = test.schema()
-    ui = AutoObject(TestAutoLogicSimple)
+    ui = AutoObject(CoreIpywidgets)
     display(ui)
-# -
 
 
 if __name__ == "__main__":
@@ -716,22 +722,3 @@ if __name__ == "__main__":
     ui.show_description = True
     ui.show_title = True
     ui.show_raw = True
-
-if __name__ == "__main__":
-    from ipyautoui.test_schema import TestEditGrid
-
-    ui = AutoObject(TestEditGrid)
-    ui.auto_open = True
-    display(ui)
-
-if __name__ == "__main__":
-    from ipyautoui.test_schema import TestAutoLogic
-
-    ui = AutoObject(TestAutoLogic)
-    display(ui)
-
-if __name__ == "__main__":
-    from ipyautoui.test_schema import TestNested
-
-    ui = AutoObject(TestNested)
-    display(ui)
