@@ -29,27 +29,20 @@ Example:
 """
 # %run __init__.py
 # #%load_ext lab_black
-import logging
+
 import pathlib
-import functools
-import ipywidgets as w
-from IPython.display import display, Markdown, clear_output, display_pretty
-from pydantic import BaseModel, Field
-from markdown import markdown
-import immutables
+from IPython.display import display
+from pydantic import BaseModel
 import json
 import traitlets as tr
 import traitlets_paths
 import typing as ty
-from enum import Enum
 
-from ipyautoui._utils import display_python_string
-from ipyautoui.custom import SaveButtonBar  #  Grid, FileChooser,
-from ipyautoui.constants import BUTTON_WIDTH_MIN
+from ipyautoui.custom import (
+    SaveButtonBar,
+)  # NOTE: removing this unused import creates circular import errors
 from ipyautoui.autoipywidget import AutoObject, get_from_schema_root
 
-# from ipyautoui.autovjsf import AutoVjsf
-
 
 # +
 
@@ -117,10 +110,11 @@ class AutoUiFileMethods(tr.HasTraits):
 
     @tr.observe("path")
     def _observe_path(self, proposal):
-        self.savebuttonbar.fns_onsave_add_action(self.file)
-        self.savebuttonbar.fns_onrevert_add_action(self.load_file)
+        self.savebuttonbar.fns_onsave_add_action(self.file, to_beginning=True)
+        self.savebuttonbar.fns_onrevert_add_action(self.load_file, to_beginning=True)
+        self.show_savebuttonbar = True
 
-    def _get_path(self, path=None):
+    def _get_path(self, path=None) -> pathlib.Path:
         if path is None:
             if self.path is not None:
                 return self.path
@@ -144,7 +138,7 @@ class AutoUiFileMethods(tr.HasTraits):
             return None
         elif v is None and p is not None and p.is_file() == True:
             # load v from p
-            return self.parse_file()
+            return self.parse_file(path=p)
         elif v is None and p is not None and p.is_file() == False:
             # v reverts to schema defaults
             return None
@@ -162,9 +156,9 @@ class AutoUiFileMethods(tr.HasTraits):
     def parse_file(self, path=None) -> dict:
         p = self._get_path(path=path)
         if p.is_file():
-            return parse_json_file(self.path, model=self.model)
+            return parse_json_file(p, model=self.model)
         else:
-            raise ValueError("path.is_file() == False")
+            raise ValueError("p.is_file() == False")
 
     def load_value(self, value, unsaved_changes=False):
         self.value = value
@@ -192,6 +186,8 @@ class AutoRenderMethods:
         schema: ty.Union[ty.Type[BaseModel], dict],
         show_raw: bool = True,
         path=None,
+        fns_onsave=None,
+        fns_onrevert=None,
     ):
         if isinstance(schema, dict):
             docstring = f"AutoRenderer for {get_from_schema_root(schema, 'title')}"
@@ -200,6 +196,9 @@ class AutoRenderMethods:
                 f"AutoRenderer for {get_from_schema_root(schema.schema(), 'title')}"
             )
 
+        # TODO: revert to using *args and **kwargs for these types of wrappers
+        #       so they only need redefining once in the main __init__
+
         class AutoRenderer(cls):
             def __init__(self, path: pathlib.Path = path):
                 f"""{docstring}"""
@@ -210,6 +209,8 @@ class AutoRenderMethods:
                     path=path,
                     value=None,
                     show_raw=show_raw,
+                    fns_onsave=fns_onsave,
+                    fns_onrevert=fns_onrevert,
                 )
 
         return AutoRenderer
@@ -220,14 +221,20 @@ class AutoRenderMethods:
         schema: ty.Union[ty.Type[BaseModel], dict],
         ext=".json",
         show_raw: bool = True,
+        fns_onsave=None,
+        fns_onrevert=None,
     ):
-        AutoRenderer = cls.create_autoui_renderer(schema, show_raw=show_raw)
-        if isinstance(schema, dict):
-            docstring = f"AutoRenderer for {get_from_schema_root(schema, 'title')}"
-        else:
-            docstring = (
-                f"AutoRenderer for {get_from_schema_root(schema.schema(), 'title')}"
-            )
+        AutoRenderer = cls.create_autoui_renderer(
+            schema, show_raw=show_raw, fns_onsave=fns_onsave, fns_onrevert=fns_onrevert
+        )
+        # if isinstance(schema, dict):
+        #     AutoRenderer.__doc__ = (
+        #         f"AutoRenderer for {get_from_schema_root(schema, 'title')}"
+        #     )
+        # else:
+        #     AutoRenderer.__doc__ = (
+        #         f"AutoRenderer for {get_from_schema_root(schema.schema(), 'title')}"
+        #     )
         return {ext: AutoRenderer}
 
 
@@ -257,6 +264,7 @@ class AutoUi(AutoObject, AutoUiFileMethods, AutoRenderMethods):
             schema, value=value, update_map_widgets=None, fdir=fdir, **kwargs
         )
         self.path = path
+        self.value = self._get_value(value, self.path)
         self.show_raw = show_raw
 
 
